@@ -1,12 +1,12 @@
 // SPDX-License-Identifier: Apache-2.0
 // 
-// Copyright 2008-2016 Conrad Sanderson (http://conradsanderson.id.au)
+// Copyright 2008-2016 Conrad Sanderson (https://conradsanderson.id.au)
 // Copyright 2008-2016 National ICT Australia (NICTA)
 // 
 // Licensed under the Apache License, Version 2.0 (the "License");
 // you may not use this file except in compliance with the License.
 // You may obtain a copy of the License at
-// http://www.apache.org/licenses/LICENSE-2.0
+// https://www.apache.org/licenses/LICENSE-2.0
 // 
 // Unless required by applicable law or agreed to in writing, software
 // distributed under the License is distributed on an "AS IS" BASIS,
@@ -24,7 +24,7 @@ template<typename eT>
 inline
 SpSubview<eT>::~SpSubview()
   {
-  arma_extra_debug_sigprint_this(this);
+  arma_debug_sigprint_this(this);
   }
 
 
@@ -40,22 +40,32 @@ SpSubview<eT>::SpSubview(const SpMat<eT>& in_m, const uword in_row1, const uword
   , n_elem(in_n_rows * in_n_cols)
   , n_nonzero(0)
   {
-  arma_extra_debug_sigprint_this(this);
+  arma_debug_sigprint_this(this);
   
   m.sync_csc();
   
-  // There must be a O(1) way to do this
-  uword lend     = m.col_ptrs[in_col1 + in_n_cols];
-  uword lend_row = in_row1 + in_n_rows;
-  uword count   = 0;
+  // count the number of non-zeros in the subview
+  uword count = 0;
   
-  for(uword i = m.col_ptrs[in_col1]; i < lend; ++i)
+  if(n_rows == m.n_rows)
     {
-    const uword m_row_indices_i = m.row_indices[i];
+    count = m.col_ptrs[aux_col1 + n_cols] - m.col_ptrs[aux_col1];
+    }
+  else
+    {
+    arma_debug_print("counting non-zeros in sparse subview");
     
-    const bool condition = (m_row_indices_i >= in_row1) && (m_row_indices_i < lend_row);
+    uword lend     = m.col_ptrs[in_col1 + in_n_cols];
+    uword lend_row = in_row1 + in_n_rows;
     
-    count += condition ? uword(1) : uword(0);
+    for(uword i = m.col_ptrs[in_col1]; i < lend; ++i)
+      {
+      const uword m_row_indices_i = m.row_indices[i];
+      
+      const bool condition = (m_row_indices_i >= in_row1) && (m_row_indices_i < lend_row);
+      
+      count += condition ? uword(1) : uword(0);
+      }
     }
   
   access::rw(n_nonzero) = count;
@@ -74,7 +84,7 @@ SpSubview<eT>::SpSubview(const SpSubview<eT>& in)
   , n_elem   (in.n_elem   )
   , n_nonzero(in.n_nonzero)
   {
-  arma_extra_debug_sigprint(arma_str::format("this = %x   in = %x") % this % &in);
+  arma_debug_sigprint(arma_str::format("this: %x; in: %x") % this % &in);
   }
 
 
@@ -90,7 +100,7 @@ SpSubview<eT>::SpSubview(SpSubview<eT>&& in)
   , n_elem   (in.n_elem   )
   , n_nonzero(in.n_nonzero)
   {
-  arma_extra_debug_sigprint(arma_str::format("this = %x   in = %x") % this % &in);
+  arma_debug_sigprint(arma_str::format("this: %x; in: %x") % this % &in);
   
   // for paranoia
   
@@ -109,7 +119,7 @@ inline
 const SpSubview<eT>&
 SpSubview<eT>::operator+=(const eT val)
   {
-  arma_extra_debug_sigprint();
+  arma_debug_sigprint();
   
   if(val == eT(0))  { return *this; }
   
@@ -127,7 +137,7 @@ inline
 const SpSubview<eT>&
 SpSubview<eT>::operator-=(const eT val)
   {
-  arma_extra_debug_sigprint();
+  arma_debug_sigprint();
   
   if(val == eT(0))  { return *this; }
   
@@ -145,9 +155,7 @@ inline
 const SpSubview<eT>&
 SpSubview<eT>::operator*=(const eT val)
   {
-  arma_extra_debug_sigprint();
-  
-  if(val == eT(0))  { (*this).zeros(); return *this; }
+  arma_debug_sigprint();
   
   if((n_elem == 0) || (n_nonzero == 0))  { return *this; }
   
@@ -207,9 +215,9 @@ inline
 const SpSubview<eT>&
 SpSubview<eT>::operator/=(const eT val)
   {
-  arma_extra_debug_sigprint();
+  arma_debug_sigprint();
   
-  arma_debug_check( (val == eT(0)), "element-wise division: division by zero" );
+  arma_conform_check( (val == eT(0)), "element-wise division: division by zero" );
   
   m.sync_csc();
   m.invalidate_cache();
@@ -268,13 +276,13 @@ inline
 const SpSubview<eT>&
 SpSubview<eT>::operator=(const Base<eT, T1>& in)
   {
-  arma_extra_debug_sigprint();
+  arma_debug_sigprint();
   
   if(is_same_type< T1, Gen<Mat<eT>, gen_zeros> >::yes)
     {
     const Proxy<T1> P(in.get_ref());
     
-    arma_debug_assert_same_size(n_rows, n_cols, P.get_n_rows(), P.get_n_cols(), "insertion into sparse submatrix");
+    arma_conform_assert_same_size(n_rows, n_cols, P.get_n_rows(), P.get_n_cols(), "insertion into sparse submatrix");
     
     (*this).zeros();
     
@@ -285,7 +293,7 @@ SpSubview<eT>::operator=(const Base<eT, T1>& in)
     {
     const Proxy<T1> P(in.get_ref());
     
-    arma_debug_assert_same_size(n_rows, n_cols, P.get_n_rows(), P.get_n_cols(), "insertion into sparse submatrix");
+    arma_conform_assert_same_size(n_rows, n_cols, P.get_n_rows(), P.get_n_cols(), "insertion into sparse submatrix");
     
     (*this).eye();
     
@@ -294,7 +302,7 @@ SpSubview<eT>::operator=(const Base<eT, T1>& in)
   
   const quasi_unwrap<T1> U(in.get_ref());
   
-  arma_debug_assert_same_size(n_rows, n_cols, U.M.n_rows, U.M.n_cols, "insertion into sparse submatrix");
+  arma_conform_assert_same_size(n_rows, n_cols, U.M.n_rows, U.M.n_cols, "insertion into sparse submatrix");
   
   spglue_merge::subview_merge(*this, U.M);
   
@@ -309,7 +317,7 @@ inline
 const SpSubview<eT>&
 SpSubview<eT>::operator+=(const Base<eT, T1>& x)
   {
-  arma_extra_debug_sigprint();
+  arma_debug_sigprint();
   
   return (*this).operator=( (*this) + x.get_ref() );
   }
@@ -322,7 +330,7 @@ inline
 const SpSubview<eT>&
 SpSubview<eT>::operator-=(const Base<eT, T1>& x)
   {
-  arma_extra_debug_sigprint();
+  arma_debug_sigprint();
   
   return (*this).operator=( (*this) - x.get_ref() );
   }
@@ -335,7 +343,7 @@ inline
 const SpSubview<eT>&
 SpSubview<eT>::operator*=(const Base<eT, T1>& x)
   {
-  arma_extra_debug_sigprint();
+  arma_debug_sigprint();
   
   SpMat<eT> tmp(*this);
   
@@ -352,9 +360,62 @@ inline
 const SpSubview<eT>&
 SpSubview<eT>::operator%=(const Base<eT, T1>& x)
   {
-  arma_extra_debug_sigprint();
+  arma_debug_sigprint();
   
-  return (*this).operator=( (*this) % x.get_ref() );
+  SpSubview<eT>& sv = (*this);
+  
+  const quasi_unwrap<T1> U(x.get_ref());
+  const Mat<eT>& B     = U.M;
+  
+  arma_conform_assert_same_size(sv.n_rows, sv.n_cols, B.n_rows, B.n_cols, "element-wise multiplication");
+  
+  SpMat<eT>& sv_m = access::rw(sv.m);
+  
+  sv_m.sync_csc();
+  sv_m.invalidate_cache();
+  
+  const uword m_row_start = sv.aux_row1;
+  const uword m_row_end   = sv.aux_row1 + sv.n_rows - 1;
+  
+  const uword m_col_start = sv.aux_col1;
+  const uword m_col_end   = sv.aux_col1 + sv.n_cols - 1;
+  
+  constexpr eT zero = eT(0);
+  
+  bool  has_zero = false;
+  uword count    = 0;
+  
+  for(uword m_col = m_col_start; m_col <= m_col_end; ++m_col)
+    {
+    const uword sv_col = m_col - m_col_start;
+    
+    const uword index_start = sv_m.col_ptrs[m_col    ];
+    const uword index_end   = sv_m.col_ptrs[m_col + 1];
+    
+    for(uword i=index_start; i < index_end; ++i)
+      {
+      const uword m_row = sv_m.row_indices[i];
+      
+      if(m_row < m_row_start)  { continue; }
+      if(m_row > m_row_end  )  { break;    }
+      
+      const uword sv_row = m_row - m_row_start;
+      
+      eT& m_val = access::rw(sv_m.values[i]);
+      
+      const eT result = m_val * B.at(sv_row, sv_col);
+      
+      m_val = result;
+      
+      if(result == zero)  { has_zero = true; } else { ++count; }
+      }
+    }
+  
+  if(has_zero)  { sv_m.remove_zeros(); }
+  
+  access::rw(sv.n_nonzero) = count;
+  
+  return (*this);
   }
 
 
@@ -365,9 +426,70 @@ inline
 const SpSubview<eT>&
 SpSubview<eT>::operator/=(const Base<eT, T1>& x)
   {
-  arma_extra_debug_sigprint();
+  arma_debug_sigprint();
   
-  return (*this).operator=( (*this) / x.get_ref() );
+  const SpSubview<eT>& A = (*this);
+  
+  const quasi_unwrap<T1> U(x.get_ref());
+  const Mat<eT>& B     = U.M;
+  
+  arma_conform_assert_same_size(A.n_rows, A.n_cols, B.n_rows, B.n_cols, "element-wise division");
+  
+  bool result_ok = true;
+  
+  constexpr eT zero = eT(0);
+  
+  const uword B_n_rows = B.n_rows;
+  const uword B_n_cols = B.n_cols;
+  
+  for(uword c=0; c < B_n_cols; ++c)
+    {
+    for(uword r=0; r < B_n_rows; ++r)
+      {
+      // a zero in B and A at the same location implies the division result is NaN;
+      // hence a zero in A (not stored) needs to be changed into a non-zero
+      
+      // for efficiency, an element in B is checked before checking the corresponding element in A
+      
+      if((B.at(r,c) == zero) && (A.at(r,c) == zero))  { result_ok = false; break; }
+      }
+    
+    if(result_ok == false)  { break; }
+    }
+  
+  if(result_ok)
+    {
+    const_iterator cit     = A.begin();
+    const_iterator cit_end = A.end();
+    
+    while(cit != cit_end)
+      {
+      const eT tmp = (*cit) / B.at(cit.row(), cit.col());
+      
+      if(tmp == zero)  { result_ok = false; break; }
+      
+      ++cit;
+      }
+    }
+  
+  if(result_ok)
+    {
+    iterator it     = (*this).begin();
+    iterator it_end = (*this).end();
+    
+    while(it != it_end)
+      {
+      (*it) /= B.at(it.row(), it.col());
+      
+      ++it;
+      }
+    }
+  else
+    {
+    (*this).operator=( (*this) / B );
+    }
+  
+  return (*this);
   }
 
 
@@ -377,7 +499,7 @@ inline
 const SpSubview<eT>&
 SpSubview<eT>::operator=(const SpSubview<eT>& x)
   {
-  arma_extra_debug_sigprint();
+  arma_debug_sigprint();
   
   return (*this).operator_equ_common(x);
   }
@@ -390,7 +512,7 @@ inline
 const SpSubview<eT>&
 SpSubview<eT>::operator=(const SpBase<eT, T1>& x)
   {
-  arma_extra_debug_sigprint();
+  arma_debug_sigprint();
   
   return (*this).operator_equ_common( x.get_ref() );
   }
@@ -403,11 +525,11 @@ inline
 const SpSubview<eT>&
 SpSubview<eT>::operator_equ_common(const SpBase<eT, T1>& in)
   {
-  arma_extra_debug_sigprint();
+  arma_debug_sigprint();
   
   const unwrap_spmat<T1> U(in.get_ref());
   
-  arma_debug_assert_same_size(n_rows, n_cols, U.M.n_rows, U.M.n_cols, "insertion into sparse submatrix");
+  arma_conform_assert_same_size(n_rows, n_cols, U.M.n_rows, U.M.n_cols, "insertion into sparse submatrix");
   
   if(U.is_alias(m))
     {
@@ -431,7 +553,7 @@ inline
 const SpSubview<eT>&
 SpSubview<eT>::operator+=(const SpBase<eT, T1>& x)
   {
-  arma_extra_debug_sigprint();
+  arma_debug_sigprint();
   
   // TODO: implement dedicated machinery
   return (*this).operator=( (*this) + x.get_ref() );
@@ -445,7 +567,7 @@ inline
 const SpSubview<eT>&
 SpSubview<eT>::operator-=(const SpBase<eT, T1>& x)
   {
-  arma_extra_debug_sigprint();
+  arma_debug_sigprint();
   
   // TODO: implement dedicated machinery
   return (*this).operator=( (*this) - x.get_ref() );
@@ -459,7 +581,7 @@ inline
 const SpSubview<eT>&
 SpSubview<eT>::operator*=(const SpBase<eT, T1>& x)
   {
-  arma_extra_debug_sigprint();
+  arma_debug_sigprint();
   
   return (*this).operator=( (*this) * x.get_ref() );
   }
@@ -472,7 +594,7 @@ inline
 const SpSubview<eT>&
 SpSubview<eT>::operator%=(const SpBase<eT, T1>& x)
   {
-  arma_extra_debug_sigprint();
+  arma_debug_sigprint();
   
   // TODO: implement dedicated machinery
   return (*this).operator=( (*this) % x.get_ref() );
@@ -486,13 +608,13 @@ inline
 const SpSubview<eT>&
 SpSubview<eT>::operator/=(const SpBase<eT, T1>& x)
   {
-  arma_extra_debug_sigprint();
+  arma_debug_sigprint();
   
   // NOTE: use of this function is not advised; it is implemented only for completeness
   
   SpProxy<T1> p(x.get_ref());
   
-  arma_debug_assert_same_size(n_rows, n_cols, p.get_n_rows(), p.get_n_cols(), "element-wise division");
+  arma_conform_assert_same_size(n_rows, n_cols, p.get_n_rows(), p.get_n_cols(), "element-wise division");
   
   if(p.is_alias(m) == false)
     {
@@ -521,7 +643,7 @@ inline
 void
 SpSubview<eT>::for_each(functor F)
   {
-  arma_extra_debug_sigprint();
+  arma_debug_sigprint();
   
   m.sync_csc();
   m.invalidate_cache();
@@ -578,7 +700,7 @@ inline
 void
 SpSubview<eT>::for_each(functor F) const
   {
-  arma_extra_debug_sigprint();
+  arma_debug_sigprint();
   
   m.sync_csc();
   
@@ -616,7 +738,7 @@ inline
 void
 SpSubview<eT>::transform(functor F)
   {
-  arma_extra_debug_sigprint();
+  arma_debug_sigprint();
   
   m.sync_csc();
   m.invalidate_cache();
@@ -672,7 +794,7 @@ inline
 void
 SpSubview<eT>::replace(const eT old_val, const eT new_val)
   {
-  arma_extra_debug_sigprint();
+  arma_debug_sigprint();
   
   if(old_val == eT(0))
     {
@@ -751,7 +873,7 @@ inline
 void
 SpSubview<eT>::clean(const typename get_pod_type<eT>::result threshold)
   {
-  arma_extra_debug_sigprint();
+  arma_debug_sigprint();
   
   if((n_elem == 0) || (n_nonzero == 0))  { return; }
   
@@ -779,16 +901,16 @@ inline
 void
 SpSubview<eT>::clamp(const eT min_val, const eT max_val)
   {
-  arma_extra_debug_sigprint();
+  arma_debug_sigprint();
   
   if(is_cx<eT>::no)
     {
-    arma_debug_check( (access::tmp_real(min_val) > access::tmp_real(max_val)), "SpSubview::clamp(): min_val must be less than max_val" );
+    arma_conform_check( (access::tmp_real(min_val) > access::tmp_real(max_val)), "SpSubview::clamp(): min_val must be less than max_val" );
     }
   else
     {
-    arma_debug_check( (access::tmp_real(min_val) > access::tmp_real(max_val)), "SpSubview::clamp(): real(min_val) must be less than real(max_val)" );
-    arma_debug_check( (access::tmp_imag(min_val) > access::tmp_imag(max_val)), "SpSubview::clamp(): imag(min_val) must be less than imag(max_val)" );
+    arma_conform_check( (access::tmp_real(min_val) > access::tmp_real(max_val)), "SpSubview::clamp(): real(min_val) must be less than real(max_val)" );
+    arma_conform_check( (access::tmp_imag(min_val) > access::tmp_imag(max_val)), "SpSubview::clamp(): imag(min_val) must be less than imag(max_val)" );
     }
   
   if((n_elem == 0) || (n_nonzero == 0))  { return; }
@@ -809,7 +931,7 @@ inline
 void
 SpSubview<eT>::fill(const eT val)
   {
-  arma_extra_debug_sigprint();
+  arma_debug_sigprint();
   
   if(val != eT(0))
     {
@@ -832,7 +954,7 @@ inline
 void
 SpSubview<eT>::zeros()
   {
-  arma_extra_debug_sigprint();
+  arma_debug_sigprint();
   
   if((n_elem == 0) || (n_nonzero == 0))  { return; }
   
@@ -889,7 +1011,7 @@ inline
 void
 SpSubview<eT>::ones()
   {
-  arma_extra_debug_sigprint();
+  arma_debug_sigprint();
   
   (*this).fill(eT(1));
   }
@@ -901,7 +1023,7 @@ inline
 void
 SpSubview<eT>::eye()
   {
-  arma_extra_debug_sigprint();
+  arma_debug_sigprint();
   
   SpMat<eT> tmp;
   
@@ -917,7 +1039,7 @@ inline
 void
 SpSubview<eT>::randu()
   {
-  arma_extra_debug_sigprint();
+  arma_debug_sigprint();
   
   Mat<eT> tmp( (*this).n_rows, (*this).n_cols, fill::randu );
   
@@ -931,7 +1053,7 @@ inline
 void
 SpSubview<eT>::randn()
   {
-  arma_extra_debug_sigprint();
+  arma_debug_sigprint();
   
   Mat<eT> tmp( (*this).n_rows, (*this).n_cols, fill::randn );
   
@@ -941,7 +1063,6 @@ SpSubview<eT>::randn()
 
 
 template<typename eT>
-arma_hot
 inline
 SpSubview_MapMat_val<eT>
 SpSubview<eT>::operator[](const uword i)
@@ -955,7 +1076,6 @@ SpSubview<eT>::operator[](const uword i)
 
 
 template<typename eT>
-arma_hot
 inline
 eT
 SpSubview<eT>::operator[](const uword i) const
@@ -969,12 +1089,11 @@ SpSubview<eT>::operator[](const uword i) const
 
 
 template<typename eT>
-arma_hot
 inline
 SpSubview_MapMat_val<eT>
 SpSubview<eT>::operator()(const uword i)
   {
-  arma_debug_check_bounds( (i >= n_elem), "SpSubview::operator(): index out of bounds" );
+  arma_conform_check_bounds( (i >= n_elem), "SpSubview::operator(): index out of bounds" );
   
   const uword lrow = i % n_rows;
   const uword lcol = i / n_rows;
@@ -985,12 +1104,11 @@ SpSubview<eT>::operator()(const uword i)
 
 
 template<typename eT>
-arma_hot
 inline
 eT
 SpSubview<eT>::operator()(const uword i) const
   {
-  arma_debug_check_bounds( (i >= n_elem), "SpSubview::operator(): index out of bounds" );
+  arma_conform_check_bounds( (i >= n_elem), "SpSubview::operator(): index out of bounds" );
   
   const uword lrow = i % n_rows;
   const uword lcol = i / n_rows;
@@ -1001,12 +1119,11 @@ SpSubview<eT>::operator()(const uword i) const
 
 
 template<typename eT>
-arma_hot
 inline
 SpSubview_MapMat_val<eT>
 SpSubview<eT>::operator()(const uword in_row, const uword in_col)
   {
-  arma_debug_check_bounds( (in_row >= n_rows) || (in_col >= n_cols), "SpSubview::operator(): index out of bounds" );
+  arma_conform_check_bounds( (in_row >= n_rows) || (in_col >= n_cols), "SpSubview::operator(): index out of bounds" );
   
   return (*this).at(in_row, in_col);
   }
@@ -1014,12 +1131,11 @@ SpSubview<eT>::operator()(const uword in_row, const uword in_col)
 
 
 template<typename eT>
-arma_hot
 inline
 eT
 SpSubview<eT>::operator()(const uword in_row, const uword in_col) const
   {
-  arma_debug_check_bounds( (in_row >= n_rows) || (in_col >= n_cols), "SpSubview::operator(): index out of bounds" );
+  arma_conform_check_bounds( (in_row >= n_rows) || (in_col >= n_cols), "SpSubview::operator(): index out of bounds" );
   
   return (*this).at(in_row, in_col);
   }
@@ -1027,7 +1143,6 @@ SpSubview<eT>::operator()(const uword in_row, const uword in_col) const
 
 
 template<typename eT>
-arma_hot
 inline
 SpSubview_MapMat_val<eT>
 SpSubview<eT>::at(const uword i)
@@ -1041,7 +1156,6 @@ SpSubview<eT>::at(const uword i)
 
 
 template<typename eT>
-arma_hot
 inline
 eT
 SpSubview<eT>::at(const uword i) const
@@ -1055,7 +1169,6 @@ SpSubview<eT>::at(const uword i) const
 
 
 template<typename eT>
-arma_hot
 inline
 SpSubview_MapMat_val<eT>
 SpSubview<eT>::at(const uword in_row, const uword in_col)
@@ -1066,7 +1179,6 @@ SpSubview<eT>::at(const uword in_row, const uword in_col)
 
 
 template<typename eT>
-arma_hot
 inline
 eT
 SpSubview<eT>::at(const uword in_row, const uword in_col) const
@@ -1132,9 +1244,9 @@ inline
 SpSubview_row<eT>
 SpSubview<eT>::row(const uword row_num)
   {
-  arma_extra_debug_sigprint();
+  arma_debug_sigprint();
   
-  arma_debug_check_bounds(row_num >= n_rows, "SpSubview::row(): out of bounds");
+  arma_conform_check_bounds(row_num >= n_rows, "SpSubview::row(): out of bounds");
   
   return SpSubview_row<eT>(const_cast< SpMat<eT>& >(m), row_num + aux_row1, aux_col1, n_cols);
   }
@@ -1146,9 +1258,9 @@ inline
 const SpSubview_row<eT>
 SpSubview<eT>::row(const uword row_num) const
   {
-  arma_extra_debug_sigprint();
+  arma_debug_sigprint();
   
-  arma_debug_check_bounds(row_num >= n_rows, "SpSubview::row(): out of bounds");
+  arma_conform_check_bounds(row_num >= n_rows, "SpSubview::row(): out of bounds");
   
   return SpSubview_row<eT>(m, row_num + aux_row1, aux_col1, n_cols);
   }
@@ -1160,9 +1272,9 @@ inline
 SpSubview_col<eT>
 SpSubview<eT>::col(const uword col_num)
   {
-  arma_extra_debug_sigprint();
+  arma_debug_sigprint();
   
-  arma_debug_check_bounds(col_num >= n_cols, "SpSubview::col(): out of bounds");
+  arma_conform_check_bounds(col_num >= n_cols, "SpSubview::col(): out of bounds");
   
   return SpSubview_col<eT>(const_cast< SpMat<eT>& >(m), col_num + aux_col1, aux_row1, n_rows);
   }
@@ -1174,9 +1286,9 @@ inline
 const SpSubview_col<eT>
 SpSubview<eT>::col(const uword col_num) const
   {
-  arma_extra_debug_sigprint();
+  arma_debug_sigprint();
   
-  arma_debug_check_bounds(col_num >= n_cols, "SpSubview::col(): out of bounds");
+  arma_conform_check_bounds(col_num >= n_cols, "SpSubview::col(): out of bounds");
   
   return SpSubview_col<eT>(m, col_num + aux_col1, aux_row1, n_rows);
   }
@@ -1188,9 +1300,9 @@ inline
 SpSubview<eT>
 SpSubview<eT>::rows(const uword in_row1, const uword in_row2)
   {
-  arma_extra_debug_sigprint();
+  arma_debug_sigprint();
   
-  arma_debug_check_bounds
+  arma_conform_check_bounds
     (
     (in_row1 > in_row2) || (in_row2 >= n_rows),
     "SpSubview::rows(): indices out of bounds or incorrectly used"
@@ -1206,9 +1318,9 @@ inline
 const SpSubview<eT>
 SpSubview<eT>::rows(const uword in_row1, const uword in_row2) const
   {
-  arma_extra_debug_sigprint();
+  arma_debug_sigprint();
   
-  arma_debug_check_bounds
+  arma_conform_check_bounds
     (
     (in_row1 > in_row2) || (in_row2 >= n_rows),
     "SpSubview::rows(): indices out of bounds or incorrectly used"
@@ -1224,9 +1336,9 @@ inline
 SpSubview<eT>
 SpSubview<eT>::cols(const uword in_col1, const uword in_col2)
   {
-  arma_extra_debug_sigprint();
+  arma_debug_sigprint();
   
-  arma_debug_check_bounds
+  arma_conform_check_bounds
     (
     (in_col1 > in_col2) || (in_col2 >= n_cols),
     "SpSubview::cols(): indices out of bounds or incorrectly used"
@@ -1242,9 +1354,9 @@ inline
 const SpSubview<eT>
 SpSubview<eT>::cols(const uword in_col1, const uword in_col2) const
   {
-  arma_extra_debug_sigprint();
+  arma_debug_sigprint();
   
-  arma_debug_check_bounds
+  arma_conform_check_bounds
     (
     (in_col1 > in_col2) || (in_col2 >= n_cols),
     "SpSubview::cols(): indices out of bounds or incorrectly used"
@@ -1260,9 +1372,9 @@ inline
 SpSubview<eT>
 SpSubview<eT>::submat(const uword in_row1, const uword in_col1, const uword in_row2, const uword in_col2)
   {
-  arma_extra_debug_sigprint();
+  arma_debug_sigprint();
   
-  arma_debug_check_bounds
+  arma_conform_check_bounds
     (
     (in_row1 > in_row2) || (in_col1 > in_col2) || (in_row2 >= n_rows) || (in_col2 >= n_cols),
     "SpSubview::submat(): indices out of bounds or incorrectly used"
@@ -1278,9 +1390,9 @@ inline
 const SpSubview<eT>
 SpSubview<eT>::submat(const uword in_row1, const uword in_col1, const uword in_row2, const uword in_col2) const
   {
-  arma_extra_debug_sigprint();
+  arma_debug_sigprint();
   
-  arma_debug_check_bounds
+  arma_conform_check_bounds
     (
     (in_row1 > in_row2) || (in_col1 > in_col2) || (in_row2 >= n_rows) || (in_col2 >= n_cols),
     "SpSubview::submat(): indices out of bounds or incorrectly used"
@@ -1296,7 +1408,7 @@ inline
 SpSubview<eT>
 SpSubview<eT>::submat(const span& row_span, const span& col_span)
   {
-  arma_extra_debug_sigprint();
+  arma_debug_sigprint();
   
   const bool row_all = row_span.whole;
   const bool col_all = row_span.whole;
@@ -1307,7 +1419,7 @@ SpSubview<eT>::submat(const span& row_span, const span& col_span)
   const uword in_col1 = col_all ? 0      : col_span.a;
   const uword in_col2 = col_all ? n_cols : col_span.b;
   
-  arma_debug_check_bounds
+  arma_conform_check_bounds
     (
     ( row_all ? false : ((in_row1 > in_row2) || (in_row2 >= n_rows)))
     ||
@@ -1325,7 +1437,7 @@ inline
 const SpSubview<eT>
 SpSubview<eT>::submat(const span& row_span, const span& col_span) const
   {
-  arma_extra_debug_sigprint();
+  arma_debug_sigprint();
   
   const bool row_all = row_span.whole;
   const bool col_all = row_span.whole;
@@ -1336,7 +1448,7 @@ SpSubview<eT>::submat(const span& row_span, const span& col_span) const
   const uword in_col1 = col_all ? 0          : col_span.a;
   const uword in_col2 = col_all ? n_cols - 1 : col_span.b;
   
-  arma_debug_check_bounds
+  arma_conform_check_bounds
     (
     ( row_all ? false : ((in_row1 > in_row2) || (in_row2 >= n_rows)))
     ||
@@ -1354,7 +1466,7 @@ inline
 SpSubview<eT>
 SpSubview<eT>::operator()(const uword row_num, const span& col_span)
   {
-  arma_extra_debug_sigprint();
+  arma_debug_sigprint();
   
   return submat(span(row_num, row_num), col_span);
   }
@@ -1366,7 +1478,7 @@ inline
 const SpSubview<eT>
 SpSubview<eT>::operator()(const uword row_num, const span& col_span) const
   {
-  arma_extra_debug_sigprint();
+  arma_debug_sigprint();
   
   return submat(span(row_num, row_num), col_span);
   }
@@ -1378,7 +1490,7 @@ inline
 SpSubview<eT>
 SpSubview<eT>::operator()(const span& row_span, const uword col_num)
   {
-  arma_extra_debug_sigprint();
+  arma_debug_sigprint();
   
   return submat(row_span, span(col_num, col_num));
   }
@@ -1390,7 +1502,7 @@ inline
 const SpSubview<eT>
 SpSubview<eT>::operator()(const span& row_span, const uword col_num) const
   {
-  arma_extra_debug_sigprint();
+  arma_debug_sigprint();
   
   return submat(row_span, span(col_num, col_num));
   }
@@ -1402,7 +1514,7 @@ inline
 SpSubview<eT>
 SpSubview<eT>::operator()(const span& row_span, const span& col_span)
   {
-  arma_extra_debug_sigprint();
+  arma_debug_sigprint();
   
   return submat(row_span, col_span);
   }
@@ -1414,7 +1526,7 @@ inline
 const SpSubview<eT>
 SpSubview<eT>::operator()(const span& row_span, const span& col_span) const
   {
-  arma_extra_debug_sigprint();
+  arma_debug_sigprint();
   
   return submat(row_span, col_span);
   }
@@ -1426,9 +1538,9 @@ inline
 void
 SpSubview<eT>::swap_rows(const uword in_row1, const uword in_row2)
   {
-  arma_extra_debug_sigprint();
+  arma_debug_sigprint();
   
-  arma_debug_check((in_row1 >= n_rows) || (in_row2 >= n_rows), "SpSubview::swap_rows(): invalid row index");
+  arma_conform_check((in_row1 >= n_rows) || (in_row2 >= n_rows), "SpSubview::swap_rows(): invalid row index");
   
   const uword lstart_col = aux_col1;
   const uword lend_col   = aux_col1 + n_cols;
@@ -1448,9 +1560,9 @@ inline
 void
 SpSubview<eT>::swap_cols(const uword in_col1, const uword in_col2)
   {
-  arma_extra_debug_sigprint();
+  arma_debug_sigprint();
   
-  arma_debug_check((in_col1 >= n_cols) || (in_col2 >= n_cols), "SpSubview::swap_cols(): invalid column index");
+  arma_conform_check((in_col1 >= n_cols) || (in_col2 >= n_cols), "SpSubview::swap_cols(): invalid column index");
   
   const uword lstart_row = aux_row1;
   const uword lend_row   = aux_row1 + n_rows;
@@ -1470,6 +1582,8 @@ inline
 typename SpSubview<eT>::iterator
 SpSubview<eT>::begin()
   {
+  m.sync_csc();
+  
   return iterator(*this);
   }
 
@@ -1642,11 +1756,10 @@ SpSubview<eT>::is_alias(const SpMat<eT>& X) const
 
 template<typename eT>
 inline
-arma_warn_unused
 eT&
 SpSubview<eT>::insert_element(const uword in_row, const uword in_col, const eT in_val)
   {
-  arma_extra_debug_sigprint();
+  arma_debug_sigprint();
   
   // This may not actually insert an element.
   const uword old_n_nonzero = m.n_nonzero;
@@ -1664,7 +1777,7 @@ inline
 void
 SpSubview<eT>::delete_element(const uword in_row, const uword in_col)
   {
-  arma_extra_debug_sigprint();
+  arma_debug_sigprint();
   
   // This may not actually delete an element.
   const uword old_n_nonzero = m.n_nonzero;
@@ -1679,7 +1792,7 @@ inline
 void
 SpSubview<eT>::invalidate_cache() const
   {
-  arma_extra_debug_sigprint();
+  arma_debug_sigprint();
   
   m.invalidate_cache();
   }
@@ -1697,7 +1810,7 @@ inline
 SpSubview_col<eT>::SpSubview_col(const SpMat<eT>& in_m, const uword in_col)
   : SpSubview<eT>(in_m, 0, in_col, in_m.n_rows, 1)
   {
-  arma_extra_debug_sigprint();
+  arma_debug_sigprint();
   }
 
 
@@ -1707,7 +1820,7 @@ inline
 SpSubview_col<eT>::SpSubview_col(const SpMat<eT>& in_m, const uword in_col, const uword in_row1, const uword in_n_rows)
   : SpSubview<eT>(in_m, in_row1, in_col, in_n_rows, 1)
   {
-  arma_extra_debug_sigprint();
+  arma_debug_sigprint();
   }
 
 
@@ -1717,7 +1830,7 @@ inline
 void
 SpSubview_col<eT>::operator=(const SpSubview<eT>& x)
   {
-  arma_extra_debug_sigprint();
+  arma_debug_sigprint();
   
   SpSubview<eT>::operator=(x);
   }
@@ -1729,7 +1842,7 @@ inline
 void
 SpSubview_col<eT>::operator=(const SpSubview_col<eT>& x)
   {
-  arma_extra_debug_sigprint();
+  arma_debug_sigprint();
   
   SpSubview<eT>::operator=(x); // interprets 'SpSubview_col' as 'SpSubview'
   }
@@ -1742,7 +1855,7 @@ inline
 void
 SpSubview_col<eT>::operator=(const SpBase<eT,T1>& x)
   {
-  arma_extra_debug_sigprint();
+  arma_debug_sigprint();
   
   SpSubview<eT>::operator=(x);
   }
@@ -1755,7 +1868,7 @@ inline
 void
 SpSubview_col<eT>::operator=(const Base<eT,T1>& x)
   {
-  arma_extra_debug_sigprint();
+  arma_debug_sigprint();
   
   SpSubview<eT>::operator=(x);
   }
@@ -1764,7 +1877,6 @@ SpSubview_col<eT>::operator=(const Base<eT,T1>& x)
 
 template<typename eT>
 inline
-arma_warn_unused
 const SpOp<SpSubview_col<eT>,spop_htrans>
 SpSubview_col<eT>::t() const
   {
@@ -1775,7 +1887,6 @@ SpSubview_col<eT>::t() const
 
 template<typename eT>
 inline
-arma_warn_unused
 const SpOp<SpSubview_col<eT>,spop_htrans>
 SpSubview_col<eT>::ht() const
   {
@@ -1786,7 +1897,6 @@ SpSubview_col<eT>::ht() const
 
 template<typename eT>
 inline
-arma_warn_unused
 const SpOp<SpSubview_col<eT>,spop_strans>
 SpSubview_col<eT>::st() const
   {
@@ -1795,6 +1905,16 @@ SpSubview_col<eT>::st() const
 
 
 
+template<typename eT>
+inline
+const SpToDOp<SpSubview_col<eT>,op_sp_as_dense>
+SpSubview_col<eT>::as_dense() const
+  {
+  return SpToDOp<SpSubview_col<eT>,op_sp_as_dense>(*this);
+  }
+
+
+
 //
 //
 //
@@ -1806,7 +1926,7 @@ inline
 SpSubview_row<eT>::SpSubview_row(const SpMat<eT>& in_m, const uword in_row)
   : SpSubview<eT>(in_m, in_row, 0, 1, in_m.n_cols)
   {
-  arma_extra_debug_sigprint();
+  arma_debug_sigprint();
   }
 
 
@@ -1816,7 +1936,7 @@ inline
 SpSubview_row<eT>::SpSubview_row(const SpMat<eT>& in_m, const uword in_row, const uword in_col1, const uword in_n_cols)
   : SpSubview<eT>(in_m, in_row, in_col1, 1, in_n_cols)
   {
-  arma_extra_debug_sigprint();
+  arma_debug_sigprint();
   }
 
 
@@ -1826,7 +1946,7 @@ inline
 void
 SpSubview_row<eT>::operator=(const SpSubview<eT>& x)
   {
-  arma_extra_debug_sigprint();
+  arma_debug_sigprint();
   
   SpSubview<eT>::operator=(x);
   }
@@ -1838,7 +1958,7 @@ inline
 void
 SpSubview_row<eT>::operator=(const SpSubview_row<eT>& x)
   {
-  arma_extra_debug_sigprint();
+  arma_debug_sigprint();
   
   SpSubview<eT>::operator=(x); // interprets 'SpSubview_row' as 'SpSubview'
   }
@@ -1851,7 +1971,7 @@ inline
 void
 SpSubview_row<eT>::operator=(const SpBase<eT,T1>& x)
   {
-  arma_extra_debug_sigprint();
+  arma_debug_sigprint();
   
   SpSubview<eT>::operator=(x);
   }
@@ -1864,7 +1984,7 @@ inline
 void
 SpSubview_row<eT>::operator=(const Base<eT,T1>& x)
   {
-  arma_extra_debug_sigprint();
+  arma_debug_sigprint();
   
   SpSubview<eT>::operator=(x);
   }
@@ -1873,7 +1993,6 @@ SpSubview_row<eT>::operator=(const Base<eT,T1>& x)
 
 template<typename eT>
 inline
-arma_warn_unused
 const SpOp<SpSubview_row<eT>,spop_htrans>
 SpSubview_row<eT>::t() const
   {
@@ -1884,7 +2003,6 @@ SpSubview_row<eT>::t() const
 
 template<typename eT>
 inline
-arma_warn_unused
 const SpOp<SpSubview_row<eT>,spop_htrans>
 SpSubview_row<eT>::ht() const
   {
@@ -1895,7 +2013,6 @@ SpSubview_row<eT>::ht() const
 
 template<typename eT>
 inline
-arma_warn_unused
 const SpOp<SpSubview_row<eT>,spop_strans>
 SpSubview_row<eT>::st() const
   {
@@ -1904,4 +2021,14 @@ SpSubview_row<eT>::st() const
 
 
 
+template<typename eT>
+inline
+const SpToDOp<SpSubview_row<eT>,op_sp_as_dense>
+SpSubview_row<eT>::as_dense() const
+  {
+  return SpToDOp<SpSubview_row<eT>,op_sp_as_dense>(*this);
+  }
+
+
+
 //! @}
